/*
 * Tapioca XMPP plugin
 * Copyright (C) 2005 INdT. 
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <glib-object.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <signal.h>
#include <sys/types.h>

#include "xmpp_connection_manager.h"

int main (int argc, char **argv)
{
    GMainLoop *mainloop;
    DBusGConnection *bus;
    DBusGProxy *proxy;
    GError *error = NULL;
    guint32 request_name_ret;
    GObject *scm;

    g_type_init ();

    if (!g_thread_supported ()) 
        g_thread_init (NULL);

    mainloop = g_main_loop_new (NULL, FALSE);

    /* Obtain a connection to the session bus */
    bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (bus == NULL) {
        g_warning ("failed to open connection to bus: %s", error->message);
        g_error_free (error);
        return 1;
    }

    proxy = dbus_g_proxy_new_for_name (bus,
            DBUS_SERVICE_DBUS,
            DBUS_PATH_DBUS,
            DBUS_INTERFACE_DBUS);

    if (!org_freedesktop_DBus_request_name (proxy, "org.tapioca.Xmpp",
                0, &request_name_ret, &error)) {
        g_warning ("failed to get name: %s", error->message);
        return 1;
    }

    if (request_name_ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
        g_warning ("got result code %u from requesting name", request_name_ret);
        g_error_free (error);
        exit (1);
    }

    scm = g_object_new (XMPP_TYPE_CONNECTION_MANAGER, NULL);
    dbus_g_connection_register_g_object (bus,
            "/org/tapioca/ConnectionManager",
            scm);
   
    g_main_loop_run (mainloop);

    return 0;    
}
