/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CALL_H__
#define __XMPP_CALL_H__

#include <talk/p2p/base/session.h>
#include <talk/base/messagequeue.h>

#include "xmpp_call_c.h"
#include "xmpp_call_c.h"
#include "xmpp_client.h"

class XmppCallSessionClient;

class XmppCall : public cricket::MessageHandler, public sigslot::has_slots<> 
{
public:
    XmppCall(XmppCallSessionClient *client, const std::string &name, const cricket::SessionID &id);
    ~XmppCall();

    bool Initiate(const std::string &to, const cricket::SessionDescription *description);
    bool Accept(const cricket::SessionDescription *description);
    bool Modify(const cricket::SessionDescription *description);
    bool Redirect(const std::string &target);
    bool Reject();
    bool Terminate();

    void AddLocalCandidates(const std::vector<cricket::Candidate> &candidate);
 
    const std::string &name() { return name_; }
    const std::string &remote_address() { return remote_address_; }
    const cricket::SessionID& id() { return id_; }
    const cricket::SessionDescription *description() { return description_; }
    const cricket::SessionDescription *remote_description() { return remote_description_; }
    cricket::Session::State state();
    cricket::Session::Error error();

    void SetDestroyedCb(xmpp_call_destroyed_cb_type cb, void *data)
        { destroyed_cb_ = cb; destroyed_cb_data_ = data; }
    void SetStateChangeCb(xmpp_call_state_change_cb_type cb, void *data)
        { state_change_cb_ = cb; state_change_cb_data_ = data; }
    void SetErrorCb(xmpp_call_error_cb_type cb, void *data)
        { error_cb_ = cb; error_cb_data_ = data; }
    void SetRemoteCapabilitiesUpdateCb(xmpp_call_remote_capabilities_updated_cb_type cb, void *data)
        { remote_capabilities_updated_cb_ = cb; remote_capabilities_updated_cb_data_ = data; }
    void SetRemoteCandidateAddedCb(xmpp_call_remote_candidate_added_cb_type cb, void *data)
        { remote_candidate_added_cb_ = cb; remote_candidate_added_cb_data_ = data; }
    
    sigslot::signal2<XmppCall *, const cricket::SessionMessage &> SignalOutgoingMessage;
    sigslot::signal2<XmppCall *, const cricket::SessionResponseMessage *> SignalSendResponse;

private:
    void SetSessionMessageFactory(cricket::SessionMessageFactory *factory);
    void SetState(cricket::Session::State state);
    void SetError(cricket::Session::Error error);

    void OnMessage(cricket::Message *pmsg);
    void OnIncomingMessage(const cricket::SessionMessage &m); 
    void OnIncomingResponse(const cricket::SessionResponseMessage &m);
    void OnIncomingError(const cricket::SessionMessage &m); 
    void OnRedirectMessage(const cricket::SessionMessage &m);

    std::string SendSessionMessage(cricket::SessionMessage::Type type,
                                   const cricket::SessionDescription *description,
                                   const std::vector<cricket::Candidate> *candidates,
                                   cricket::SessionMessage::Cookie *redirect_cookie);
    void SendInitiateResponseMessage(const cricket::SessionMessage *message); 
    void SendAcknowledgementMessage(const cricket::SessionMessage *message); 

    cricket::CriticalSection crit_;
    XmppCallSessionClient *client_;
    bool initiator_;
    cricket::SessionMessageFactory *session_message_factory_;
    std::string name_;
    std::string remote_address_;
    cricket::SessionID id_;
    const cricket::SessionDescription *description_;
    const cricket::SessionDescription *remote_description_;
    cricket::Session::State state_;
    cricket::Session::Error error_;
    std::string redirect_target_;
    std::string initiate_msgid_;

    xmpp_call_destroyed_cb_type destroyed_cb_;
    void *destroyed_cb_data_;
    xmpp_call_state_change_cb_type state_change_cb_;
    void *state_change_cb_data_;
    xmpp_call_error_cb_type error_cb_;
    void *error_cb_data_;
    xmpp_call_remote_capabilities_updated_cb_type remote_capabilities_updated_cb_;
    void *remote_capabilities_updated_cb_data_;
    xmpp_call_remote_candidate_added_cb_type remote_candidate_added_cb_;
    void *remote_candidate_added_cb_data_;
    
    friend class XmppCallSessionClient;
};

#endif
