/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_call_c.h"
#include "xmpp_call.h"

#include <talk/p2p/base/sessiondescription.h>

#include <stdarg.h>

void
xmpp_call_initiate (xmpp_call_t call,
                    const char *jid,
                    xmpp_call_session_description_t session_description)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Initiate (std::string (jid), reinterpret_cast<cricket::SessionDescription *>(session_description));
}

void
xmpp_call_accept (xmpp_call_t call,
                  xmpp_call_session_description_t session_description)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Accept (reinterpret_cast<cricket::SessionDescription *>(session_description));
}

void
xmpp_call_modify (xmpp_call_t call,
                  xmpp_call_session_description_t session_description)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Modify (reinterpret_cast<cricket::SessionDescription *>(session_description));
}

void
xmpp_call_redirect (xmpp_call_t call,
                    const char *jid)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Redirect (std::string (jid));
}

void
xmpp_call_reject (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Reject ();
}

void
xmpp_call_terminate (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->Terminate ();
}

void 
xmpp_call_add_local_candidates(xmpp_call_t call,
                               xmpp_candidate_t first_candidate,
                               ...)
{
    if (!first_candidate)
        return;
    
    va_list ap;
    va_start (ap, first_candidate);
    
    std::vector<cricket::Candidate> candidates;
    xmpp_candidate_t candidate = first_candidate;
    while (candidate) {
        candidates.push_back (*reinterpret_cast<cricket::Candidate *>(candidate));
        candidate = va_arg (ap, xmpp_candidate_t);
    }

    va_end (ap);

    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->AddLocalCandidates (candidates); 
}

const char *
xmpp_call_get_name (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    return xc->name().c_str();
}

const char *
xmpp_call_get_remote_address (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    return xc->remote_address().c_str();
}

xmpp_call_session_description_t 
xmpp_call_get_description (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    return (xmpp_call_session_description_t) xc->description();
}

xmpp_call_session_description_t 
xmpp_call_get_remote_description (xmpp_call_t call)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    return (xmpp_call_session_description_t) xc->remote_description();
}

void
xmpp_call_set_destroyed_cb (xmpp_call_t call, 
                            xmpp_call_destroyed_cb_type cb,
                            void *cb_data)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->SetDestroyedCb (cb, cb_data);
}

void
xmpp_call_set_state_change_cb (xmpp_call_t call, 
                               xmpp_call_state_change_cb_type cb,
                               void *cb_data)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->SetStateChangeCb (cb, cb_data);
}

void 
xmpp_call_set_error_cb (xmpp_call_t call, 
                        xmpp_call_error_cb_type cb,
                        void *cb_data)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->SetErrorCb (cb, cb_data);
}

void 
xmpp_call_set_remote_capabilities_updated_cb (xmpp_call_t call, 
                                              xmpp_call_remote_capabilities_updated_cb_type cb,
                                              void *cb_data)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->SetRemoteCapabilitiesUpdateCb (cb, cb_data);
}

void 
xmpp_call_set_remote_candidate_added_cb (xmpp_call_t call, 
                                         xmpp_call_remote_candidate_added_cb_type cb,
                                         void *cb_data)
{
    XmppCall *xc = reinterpret_cast<XmppCall *>(call);
    xc->SetRemoteCandidateAddedCb (cb, cb_data);
}
