/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CALL_C_H__
#define __XMPP_CALL_C_H__

#include "xmpp_call_session_client_c.h"
#include "xmpp_call_session_description_c.h"
#include "xmpp_candidate_c.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void (* xmpp_call_destroyed_cb_type)    (xmpp_call_t call, 
                                                 void *data);
typedef void (* xmpp_call_state_change_cb_type) (xmpp_call_t call, 
                                                 void *data, 
                                                 int state);
typedef void (* xmpp_call_error_cb_type)        (xmpp_call_t call, 
                                                 void *data, 
                                                 int error);
typedef void (* xmpp_call_remote_capabilities_updated_cb_type)
                                                (xmpp_call_t call, 
                                                 void *data, 
                                                 xmpp_call_session_description_t session_description);
typedef void (* xmpp_call_remote_candidate_added_cb_type)
                                                (xmpp_call_t call, 
                                                 void *data, 
                                                 xmpp_candidate_t candidate);

void        xmpp_call_initiate               (xmpp_call_t call,
                                              const char *jid,
                                              xmpp_call_session_description_t session_description);
void        xmpp_call_accept                 (xmpp_call_t call,
                                              xmpp_call_session_description_t session_description);
void        xmpp_call_modify                 (xmpp_call_t call,
                                              xmpp_call_session_description_t session_description);
void        xmpp_call_redirect               (xmpp_call_t call,
                                              const char *jid);
void        xmpp_call_reject                 (xmpp_call_t call);
void        xmpp_call_terminate              (xmpp_call_t call);

void        xmpp_call_add_local_candidates   (xmpp_call_t call,
                                              xmpp_candidate_t candidate,
                                              ...);

const char *xmpp_call_get_name               (xmpp_call_t call);
const char *xmpp_call_get_remote_address     (xmpp_call_t call);
xmpp_call_session_description_t 
            xmpp_call_get_description        (xmpp_call_t call);
xmpp_call_session_description_t 
            xmpp_call_get_remote_description (xmpp_call_t call);

void        xmpp_call_set_destroyed_cb       (xmpp_call_t call, 
                                              xmpp_call_destroyed_cb_type cb,
                                              void *cb_data);
void        xmpp_call_set_state_change_cb    (xmpp_call_t call, 
                                              xmpp_call_state_change_cb_type cb,
                                              void *cb_data);
void        xmpp_call_set_error_cb           (xmpp_call_t call, 
                                              xmpp_call_error_cb_type cb,
                                              void *cb_data);
void        xmpp_call_set_remote_capabilities_updated_cb 
                                             (xmpp_call_t call, 
                                              xmpp_call_remote_capabilities_updated_cb_type cb,
                                              void *cb_data);
void        xmpp_call_set_remote_candidate_added_cb 
                                             (xmpp_call_t call, 
                                              xmpp_call_remote_candidate_added_cb_type cb,
                                              void *cb_data);

#ifdef __cplusplus
}
#endif

#endif
