/*
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CALL_SESSION_CLIENT_H__
#define __XMPP_CALL_SESSION_CLIENT_H__

#include <talk/base/criticalsection.h>
#include <talk/base/sigslot.h>
#include <talk/p2p/client/xmppsessionmessage.h>
#include <talk/xmpp/xmppengine.h>
#include <talk/xmpp/xmpptask.h>

#include <map>

#include "xmpp_message.h"

namespace buzz {
    class XmlElement;
}

class XmppCall;
class XmppCallSessionMessageFactory;

class XmppCallSessionClient : public buzz::XmppTask 
{
public:
    XmppCallSessionClient(buzz::Task *parent);
    ~XmppCallSessionClient();
    
    XmppCall *CreateCall();
    void DestroyCall(XmppCall *call);
    void TerminateAll();
    XmppCall *GetCall(const cricket::SessionID& id); 
    
    int ProcessStart();

    sigslot::signal2<XmppCall *, bool> SignalCallCreate;
    sigslot::signal1<XmppCall *> SignalCallDestroy;

protected:
    bool HandleStanza(const buzz::XmlElement *stanza);

private:
    XmppCall *CreateCall(const std::string &name, 
                         const cricket::SessionID &id, 
                         bool received_initiate);

    bool ParseIncomingMessage(const buzz::XmlElement *stanza,
                              cricket::XmppSessionMessage& message);
    bool ParseIncomingResponse(const buzz::XmlElement *stanza,
                               cricket::XmppSessionResponseMessage& response);
    void ParseHeader(const buzz::XmlElement *stanza, 
                     cricket::XmppSessionMessage &message);
    void ParseInitiateAcceptModify(const buzz::XmlElement *stanza,
                                   cricket::XmppSessionMessage &message);
    void ParseCandidates(const buzz::XmlElement *stanza, 
                         cricket::XmppSessionMessage &message);
    bool ParseCandidate(const buzz::XmlElement *child,
                        cricket::Candidate *candidate);
    void ParseRedirect(const buzz::XmlElement *stanza, 
                       cricket::XmppSessionMessage &message);
    void ParseRejectTerminate(const buzz::XmlElement *stanza, 
                              cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateHeader(const cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateCandidate(const cricket::Candidate &candidate);
    buzz::XmlElement *TranslateInitiateAcceptModify(const cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateCandidates(const cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateRejectTerminate(const cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateRedirect(const cricket::XmppSessionMessage &message);
    buzz::XmlElement *TranslateSessionDescription(const cricket::SessionDescription *description);
    const cricket::SessionDescription *CreateSessionDescription(const buzz::XmlElement *element);
    void HandleIncomingMessage(const cricket::SessionMessage &m); 
    void OnSendResponse(XmppCall *call,  
                        const cricket::SessionResponseMessage *message);
    void OnOutgoingMessage(XmppCall *call,
                           const cricket::SessionMessage &message);
    void OnIncomingResponse(const buzz::XmlElement *original_stanza,
                            const buzz::XmlElement *response_stanza); 
    void OnFailedSend(const buzz::XmlElement *original_stanza,
                      const buzz::XmlElement *failure_stanza); 

    cricket::CriticalSection crit_;
    std::map<cricket::SessionID, XmppCall *> calls_;
    XmppCallSessionMessageFactory *session_message_factory_;

    friend class XmppCallSessionSendTask;
};

#endif 
