/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_candidate_c.h"

#include <talk/p2p/base/candidate.h>

#define CANDIDATE(candidate) reinterpret_cast<cricket::Candidate *>(candidate)

xmpp_candidate_t xmpp_candidate_new                 (void) { return (xmpp_candidate_t) new cricket::Candidate(); }
void             xmpp_candidate_destroy             (xmpp_candidate_t candidate) { delete CANDIDATE (candidate); }

void             xmpp_candidate_set_name            (xmpp_candidate_t candidate, const char *name) { CANDIDATE (candidate)->set_name (name); }
void             xmpp_candidate_set_protocol        (xmpp_candidate_t candidate, const char *protocol) { CANDIDATE (candidate)->set_protocol (protocol); }
void             xmpp_candidate_set_address         (xmpp_candidate_t candidate, const char *ip, unsigned int port) 
{ 
    cricket::SocketAddress address;
    address.SetIP(ip);
    address.SetPort(port);

    CANDIDATE (candidate)->set_address (address); 
}

void             xmpp_candidate_set_preference      (xmpp_candidate_t candidate, float preference) { CANDIDATE (candidate)->set_preference (preference); }
void             xmpp_candidate_set_preference_str  (xmpp_candidate_t candidate, const char *preference) { CANDIDATE (candidate)->set_preference_str (preference); }
void             xmpp_candidate_set_username        (xmpp_candidate_t candidate, const char *username) { CANDIDATE (candidate)->set_username (username); }
void             xmpp_candidate_set_password        (xmpp_candidate_t candidate, const char *password) { CANDIDATE (candidate)->set_password (password); }
void             xmpp_candidate_set_type            (xmpp_candidate_t candidate, const char *type) { CANDIDATE (candidate)->set_type (type); }
void             xmpp_candidate_set_network_name    (xmpp_candidate_t candidate, const char *network_name) { CANDIDATE (candidate)->set_network_name (network_name); }
void             xmpp_candidate_set_generation      (xmpp_candidate_t candidate, unsigned int generation) { CANDIDATE (candidate)->set_generation (generation); }
void             xmpp_candidate_set_generation_str  (xmpp_candidate_t candidate, const char *generation) { CANDIDATE (candidate)->set_generation_str (generation); }

const char      *xmpp_candidate_get_name            (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->name ().c_str(); }
const char      *xmpp_candidate_get_protocol        (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->protocol ().c_str(); }
void             xmpp_candidate_get_address         (xmpp_candidate_t candidate, const char **ip, unsigned int *port) 
{    
    *ip = CANDIDATE (candidate)->address().IPAsString().c_str();
    *port = CANDIDATE (candidate)->address().port();
}

float            xmpp_candidate_get_preference      (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->preference (); }
const char      *xmpp_candidate_get_preference_str  (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->preference_str ().c_str(); }
const char      *xmpp_candidate_get_username        (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->username ().c_str(); }
const char      *xmpp_candidate_get_password        (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->password ().c_str(); }
const char      *xmpp_candidate_get_type            (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->type ().c_str(); }
const char      *xmpp_candidate_get_network_name    (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->network_name ().c_str(); }
unsigned int     xmpp_candidate_get_generation      (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->generation (); }
const char      *xmpp_candidate_get_generation_str  (xmpp_candidate_t candidate) { return CANDIDATE (candidate)->generation_str ().c_str(); }
