/*
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CHAT_H__
#define __XMPP_CHAT_H__

#include <talk/base/criticalsection.h>
#include <talk/base/sigslot.h>
#include <talk/xmpp/jid.h>

#include <iostream>

#include "xmpp_chat_c.h"
#include "xmpp_message.h"

class XmppChatSessionClient;

class XmppChat : public sigslot::has_slots<> {
public:
    XmppChat(XmppChatSessionClient *session_client, const buzz::Jid &remote_jid);
    ~XmppChat();

    const buzz::Jid &remote_jid() { return remote_jid_; }

    void SendMessage(const std::string &subject,
                     const std::string &body);

    void SetEstablishedCb(xmpp_chat_established_cb_type cb,
                          void *data)
        { established_cb_ = cb; established_cb_data_ = data; }
    void SetMessageReceivedCb(xmpp_chat_message_received_cb_type cb,
                              void *data)
        { message_received_cb_ = cb; message_received_cb_data_ = data; }
    void SetDeliveryErrorCb(xmpp_chat_delivery_error_cb_type cb,
                            void *data)
        { delivery_error_cb_ = cb; delivery_error_cb_data_ = data; }

private:
    void SetRemoteJid(const buzz::Jid &jid);
    void OnMessage(const XmppMessage &message);
    void OnError(const XmppMessage &message, int error_code);
    
    cricket::CriticalSection crit_;
    XmppChatSessionClient *session_client_;
    buzz::Jid remote_jid_;
    bool established;
    xmpp_chat_established_cb_type established_cb_;
    void *established_cb_data_;
    xmpp_chat_message_received_cb_type message_received_cb_;
    void *message_received_cb_data_;
    xmpp_chat_delivery_error_cb_type delivery_error_cb_;
    void *delivery_error_cb_data_;
    
    friend class XmppChatSessionClient;
};

#endif
