/*
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CHAT_SESSION_CLIENT_H__
#define __XMPP_CHAT_SESSION_CLIENT_H__

#include <talk/base/criticalsection.h>
#include <talk/base/sigslot.h>
#include <talk/xmpp/xmppengine.h>
#include <talk/xmpp/xmpptask.h>

#include <map>

#include "xmpp_message.h"

namespace buzz {
    class XmlElement;
}

class XmppChat;

class XmppChatSessionClient : public buzz::XmppTask {
public:
    XmppChatSessionClient(buzz::Task *parent);
    ~XmppChatSessionClient();
    
    XmppChat *CreateChat(const buzz::Jid &remote_jid, 
                         bool incoming = false);
    void DestroyChat(XmppChat *chat);
    
    int ProcessStart();

    sigslot::signal2<XmppChat *, bool> SignalChatCreate;
    sigslot::signal1<XmppChat *> SignalChatDestroy;

protected:
    bool HandleStanza(const buzz::XmlElement *stanza);

private:
    XmppMessage *CreateMessage(const buzz::XmlElement *element);
    buzz::XmlElement *TranslateMessage(const XmppMessage *message);

    cricket::CriticalSection crit_;
    std::map<std::string, XmppChat *> chats_;

    friend class XmppChat;
};

#endif 
