/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CLIENT_H__
#define __XMPP_CLIENT_H__

#include "xmpp_client_c.h"

#include "xmpppump.h"
#include "status.h"

#include <talk/base/thread.h>
#include <talk/p2p/base/session.h>
#include <talk/p2p/client/sessionclient.h>
#include <talk/xmpp/xmppclient.h>
#include <talk/xmpp/xmppclientsettings.h>

namespace cricket {
    class Thread;
    class NetworkManager;
    class PortAllocator;
    class Receiver;
    class FakeMediaEngine;
}

namespace buzz {
    class PresenceOutTask;
}

class XmppCallSessionClient;
class XmppCall;
class XmppChatSessionClient;
class XmppChat;
class XmppPresencePush;
class XmppRoster;

class XmppClient : public sigslot::has_slots<>, XmppPumpNotify {
public:
    XmppClient();
    ~XmppClient();
   
    void Connect(const buzz::XmppClientSettings &xcs);
    void Disconnect();

    void SendRaw(const std::string &data);
    
    cricket::Thread *worker_thread() { return worker_thread_; }
    buzz::XmppClient *client() { return pump_->client(); }
    XmppCallSessionClient *call_client() { return call_client_; }
    XmppChatSessionClient *chat_client() { return chat_client_; }
    buzz::PresenceOutTask *presence_out() { return presence_out_; }
    XmppPresencePush *presence_push() { return presence_push_; }
    XmppRoster *roster() { return roster_; }
    
    void SetStateChangeCb(xmpp_client_state_change_cb_type cb,
                          void *data)
        { state_change_cb_ = cb, state_change_cb_data_ = data; }
    void SetIncomingCallCb(xmpp_client_incoming_call_cb_type cb,
                           void *data)
        { incoming_call_cb_ = cb, incoming_call_cb_data_ = data; }
    void SetIncomingChatCb(xmpp_client_incoming_chat_cb_type cb,
                           void *data)
        { incoming_chat_cb_ = cb, incoming_chat_cb_data_ = data; }
    
private:
    void OnStateChange(buzz::XmppEngine::State state);
    void OnChatCreate(XmppChat* chat,
                      bool incoming);
    void OnCallCreate(XmppCall* call,
                      bool incoming);
    void InitChatClient();
    void InitCallClient();
    void InitPresence();
    void InitRoster();
    void OnStatusUpdate(const buzz::Status& status);

    XmppPump *pump_;
    cricket::FakeMediaEngine *media_engine_;
    cricket::Thread *worker_thread_;
    cricket::NetworkManager *network_manager_;
    cricket::PortAllocator *port_allocator_;
    cricket::SessionManager *session_manager_;
    XmppCallSessionClient *call_client_;
    XmppChatSessionClient *chat_client_;
    cricket::Receiver *receiver_;
    buzz::PresenceOutTask *presence_out_;
    XmppPresencePush *presence_push_;
    XmppRoster *roster_;

    xmpp_client_state_change_cb_type state_change_cb_;
    void *state_change_cb_data_;
    xmpp_client_incoming_call_cb_type incoming_call_cb_;
    void *incoming_call_cb_data_;
    xmpp_client_incoming_chat_cb_type incoming_chat_cb_;
    void *incoming_chat_cb_data_;
};

#endif
