/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_client_c.h"
#include "xmpp_client.h"

#include <talk/base/xmpppassword.h>
#include <talk/xmpp/xmppclientsettings.h>

class InsecurePasswordImpl : public buzz::XmppPasswordImpl {
public:
    std::string& password() { return password_; }
    const std::string& password() const { return password_; }

    InsecurePasswordImpl() {}
    InsecurePasswordImpl(const char *password) { password_ = password; }
    virtual ~InsecurePasswordImpl() {}
    virtual size_t GetLength() const { return password_.size(); }
    virtual void CopyTo(char * dest, bool nullterminate) const 
    {
        memcpy(dest, password_.data(), password_.size());
        if (nullterminate) dest[password_.size()] = 0;
    }
    virtual std::string UrlEncode() const { return password_; }
    virtual XmppPasswordImpl * Copy() const 
    {
        InsecurePasswordImpl * copy = new InsecurePasswordImpl;
        copy->password() = password_;
        return copy;
    }
    
private:
    std::string password_;
};

xmpp_client_t
xmpp_client_new (void)
{
    XmppClient *client = new XmppClient();
    return (xmpp_client_t) client;
}
    
void 
xmpp_client_connect (xmpp_client_t client,
                     const char *user,
                     const char *host,
                     const char *resource,
                     const char *pass,
                     const char *server,
                     unsigned int server_port,
                     int use_proxy,
                     int proxy_type,
                     const char *proxy,
                     unsigned int proxy_port,
                     const char *proxy_user,
                     const char *proxy_pass,
                     int use_tls)
{
    buzz::XmppClientSettings xcs;
    InsecurePasswordImpl insecure_pass (pass);

    printf("user=%s host=%s pass=%s use_tls=%s server=%s server_port=%d\n", 
           user, host, pass, (use_tls ? "true" : "false"), server, server_port);

    xcs.set_user (user);
    xcs.set_host (host);
    if (resource)
        xcs.set_resource (resource);
    xcs.set_use_tls ((bool) use_tls);
    xcs.set_pass (buzz::XmppPassword (insecure_pass));

    if (use_proxy) {
        xcs.set_proxy ((cricket::ProxyType) proxy_type);
        xcs.set_proxy_host (proxy);
        xcs.set_proxy_port (proxy_port);

        if (proxy_user) {
            InsecurePasswordImpl insecure_proxy_pass (proxy_pass);
            xcs.set_use_proxy_auth (true);
            xcs.set_proxy_user (proxy_user);
            xcs.set_proxy_pass (buzz::XmppPassword (insecure_proxy_pass));
        }
    }
    
    xcs.set_server (cricket::SocketAddress (server, server_port));
    
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->Connect (xcs);
}

void
xmpp_client_disconnect (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->Disconnect ();
}

void
xmpp_client_send_raw (xmpp_client_t client,
                      const char *data)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->SendRaw (data); 
}

xmpp_thread_t
xmpp_client_get_worker_thread (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_thread_t) xc->worker_thread();
}

xmpp_call_session_client_t
xmpp_client_get_call_session_client (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_call_session_client_t) xc->call_client();
}

xmpp_chat_session_client_t
xmpp_client_get_chat_session_client (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_chat_session_client_t) xc->chat_client();
}

xmpp_presence_out_t
xmpp_client_get_presence_out (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_presence_out_t) xc->presence_out();
}

xmpp_presence_push_t
xmpp_client_get_presence_push (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_presence_push_t) xc->presence_push();
}

xmpp_roster_t
xmpp_client_get_roster (xmpp_client_t client)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    return (xmpp_roster_t) xc->roster();
}

void
xmpp_client_set_state_change_cb (xmpp_client_t client, 
                                 xmpp_client_state_change_cb_type cb,
                                 void *data)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->SetStateChangeCb (cb, data);
}

void
xmpp_client_set_incoming_call_cb (xmpp_client_t client, 
                                  xmpp_client_incoming_call_cb_type cb,
                                  void *data)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->SetIncomingCallCb (cb, data);
}

void
xmpp_client_set_incoming_chat_cb (xmpp_client_t client, 
                                  xmpp_client_incoming_chat_cb_type cb,
                                  void *data)
{
    XmppClient *xc = reinterpret_cast<XmppClient *>(client);
    xc->SetIncomingChatCb (cb, data);
}
