/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_CLIENT_C_H__
#define __XMPP_CLIENT_C_H__

#include "xmpp_call_c.h"
#include "xmpp_call_session_client_c.h"
#include "xmpp_chat_c.h"
#include "xmpp_chat_session_client_c.h"
#include "xmpp_presence_out_c.h"
#include "xmpp_presence_push_c.h"
#include "xmpp_roster_c.h"
#include "xmpp_thread_c.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void* xmpp_client_t;

typedef void (* xmpp_client_state_change_cb_type)  (xmpp_client_t client, 
                                                    void *data, 
                                                    int state,
                                                    int error);
typedef void (* xmpp_client_incoming_call_cb_type) (xmpp_client_t client, 
                                                    void *data, 
                                                    const char *from,
                                                    xmpp_call_t call);
typedef void (* xmpp_client_incoming_chat_cb_type) (xmpp_client_t client, 
                                                    void *data, 
                                                    const char *from,
                                                    xmpp_chat_t chat);

xmpp_client_t xmpp_client_new                      (void);

void          xmpp_client_connect                  (xmpp_client_t client,
                                                    const char *user,
                                                    const char *host,
                                                    const char *resource,
                                                    const char *pass,
                                                    const char *server,
                                                    unsigned int server_port,
                                                    int use_proxy,
                                                    int proxy_type,
                                                    const char *proxy,
                                                    unsigned int proxy_port,
                                                    const char *proxy_user,
                                                    const char *proxy_pass,
                                                    int use_tls);
void          xmpp_client_disconnect               (xmpp_client_t client);

void          xmpp_client_send_raw                 (xmpp_client_t client, 
                                                    const char *data);

xmpp_thread_t
              xmpp_client_get_worker_thread        (xmpp_client_t client);
xmpp_call_session_client_t
              xmpp_client_get_call_session_client  (xmpp_client_t client);
xmpp_chat_session_client_t
              xmpp_client_get_chat_session_client  (xmpp_client_t client);
xmpp_presence_out_t
              xmpp_client_get_presence_out         (xmpp_client_t client);
xmpp_presence_push_t
              xmpp_client_get_presence_push        (xmpp_client_t client);
xmpp_roster_t
              xmpp_client_get_roster               (xmpp_client_t client);

void          xmpp_client_set_state_change_cb      (xmpp_client_t client, 
                                                    xmpp_client_state_change_cb_type cb,
                                                    void *data);
void          xmpp_client_set_incoming_call_cb     (xmpp_client_t client, 
                                                    xmpp_client_incoming_call_cb_type cb,
                                                    void *data);
void          xmpp_client_set_incoming_chat_cb     (xmpp_client_t client, 
                                                    xmpp_client_incoming_chat_cb_type cb,
                                                    void *data);

#ifdef __cplusplus
}
#endif

#endif
