/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_presence_push.h"

#include "presencepushtask.h"
#include "status.h"

#include <talk/xmpp/xmppclient.h>

XmppPresencePush::XmppPresencePush(buzz::XmppClient *client) 
    : status_updated_cb_(NULL),
      status_updated_cb_data_(NULL)
{
    presence_push_ = new buzz::PresencePushTask(client);
    presence_push_->SignalStatusUpdate.connect(
            this, &XmppPresencePush::OnStatusUpdate);
    presence_push_->Start();
}

void XmppPresencePush::RequestStatus(const buzz::Jid &jid)
{
    bool found = false;

    cricket::CritScope s(&crit_);

    std::map<std::string, RosterItem>::iterator iter = roster_.begin();
    while (iter != roster_.end()) {
        RosterItem &item = iter->second;
        if (item.jid == jid ||
            item.jid.BareJid() == jid) {
            if (status_updated_cb_)
                status_updated_cb_((xmpp_presence_push_t) this, status_updated_cb_data_, 
                        item.jid.Str().c_str(), item.show, item.status.c_str(), 
                        item.phone_capability);

            found = true;
        }
        ++iter;
    }    

    if (!found) {
        status_updated_cb_((xmpp_presence_push_t) this, status_updated_cb_data_, 
                jid.Str().c_str(), buzz::Status::SHOW_OFFLINE, NULL, false);
    }
}

bool XmppPresencePush::has_phone_capability(const buzz::Jid &jid)
{
    cricket::CritScope s(&crit_);
    std::map<std::string, RosterItem>::iterator iter = roster_.find(jid.Str());
    if (iter != roster_.end())
        return iter->second.phone_capability;
    return false;
}

void XmppPresencePush::OnStatusUpdate(const buzz::Status& status) 
{
    cricket::CritScope s(&crit_);
    
    std::string key = status.jid().Str();
    if (status.available()) {
        RosterItem *item;

        std::map<std::string, RosterItem>::iterator iter = roster_.find(key);
        if (iter != roster_.end()) {
            item = &iter->second;
            if (item->jid == status.jid() &&
                item->show == status.show() &&
                item->status == status.status() &&
                item->phone_capability == status.phone_capability()) {
                return;
            }
        }
        
        item = &roster_[key];
        item->jid = status.jid();
        item->show = status.show();
        item->status = status.status();
        item->phone_capability = status.phone_capability();
    } 
    else {
        std::map<std::string, RosterItem>::iterator iter = roster_.find(key);
        if (iter != roster_.end())
            roster_.erase(iter);
    }
    
    if (status_updated_cb_)
        status_updated_cb_((xmpp_presence_push_t) this, status_updated_cb_data_, 
                status.jid().Str().c_str(), status.show(), status.status().c_str(), 
                status.phone_capability());
}
