/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_PRESENCE_PUSH_H__
#define __XMPP_PRESENCE_PUSH_H__

#include "xmpp_presence_push_c.h"

#include "status.h"

#include <talk/base/sigslot.h>
#include <talk/base/criticalsection.h>

#include <map>

namespace buzz {
    class XmppClient;
    class PresencePushTask;
}

class XmppPresencePush : public sigslot::has_slots<> {
public:
    XmppPresencePush(buzz::XmppClient *client);
  
    void RequestStatus(const buzz::Jid &jid);
    bool has_phone_capability(const buzz::Jid &jid);

    void SetStatusUpdatedCb(xmpp_presence_push_status_updated_cb_type cb,
                           void *data)
        { status_updated_cb_ = cb; status_updated_cb_data_ = data; }

protected:
    struct RosterItem {
        buzz::Jid jid;
        buzz::Status::Show show;
        std::string status;
        bool phone_capability;
    };

    void OnStatusUpdate(const buzz::Status& status);
  
    cricket::CriticalSection crit_; 
    std::map<std::string, RosterItem> roster_;
    buzz::PresencePushTask *presence_push_;
    xmpp_presence_push_status_updated_cb_type status_updated_cb_;
    void *status_updated_cb_data_;
};

#endif
