/*
 * Copyright (c) 2005 INdT.
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_ROSTER_H__
#define __XMPP_ROSTER_H__

#include <talk/base/criticalsection.h>
#include <talk/base/sigslot.h>
#include <talk/xmpp/xmppengine.h>
#include <talk/xmpp/xmpptask.h>
#include <talk/xmpp/jid.h>

#include <map>
#include <string>

#include "xmpp_roster_c.h"

namespace buzz {
    class XmlElement;
}

class XmppRoster : public buzz::XmppTask {
public:
    XmppRoster(buzz::Task *parent);
    ~XmppRoster();
        
    int ProcessStart();
    void AddContact(const buzz::Jid &jid, const std::string &name, 
        const std::string &group);
    void RemoveContact(const buzz::Jid &jid);
    void AuthorizeContact(const buzz::Jid &jid);
    void DenyContact(const buzz::Jid &jid);
    void RequestContacts();
    void RequestInfo(const buzz::Jid &jid);

    void SetDenyCb(xmpp_roster_deny_authorize_cb_type cb, void *data)
        { deny_cb_ = cb; deny_data_ = data; }
    void SetAuthorizeCb(xmpp_roster_deny_authorize_cb_type cb, void *data)
        { authorize_cb_ = cb; authorize_data_ = data; }
    void SetRequestCb(xmpp_roster_deny_authorize_cb_type cb, void *data)
        { request_cb_ = cb; request_data_ = data; }
    void SetContactCb(xmpp_roster_contact_retrieved_cb_type cb, void *data)
        { contact_cb_ = cb; contact_data_ = data; }
    void SetInfoCb(xmpp_roster_info_retrieved_cb_type cb, void *data)
        { info_cb_ = cb; info_data_ = data; }
private:
    bool HandleStanza(const buzz::XmlElement *stanza);
    
    xmpp_roster_deny_authorize_cb_type deny_cb_;
    void *deny_data_;
    xmpp_roster_deny_authorize_cb_type authorize_cb_;
    void *authorize_data_;
    xmpp_roster_deny_authorize_cb_type request_cb_;
    void *request_data_;
    xmpp_roster_contact_retrieved_cb_type contact_cb_;
    void *contact_data_;
    xmpp_roster_info_retrieved_cb_type info_cb_;
    void *info_data_;
    cricket::CriticalSection crit_;
};

#endif 
