/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_roster_c.h"
#include "xmpp_roster.h"

#include <talk/xmpp/xmppclient.h>

void        
xmpp_roster_deny_contact (xmpp_roster_t roster,
                          const char *jid)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->DenyContact (buzz::Jid(jid));
}

void
xmpp_roster_authorize_contact (xmpp_roster_t roster,
                               const char *jid)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->AuthorizeContact (buzz::Jid(jid));
}

void
xmpp_roster_request_contacts (xmpp_roster_t roster)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->RequestContacts ();
}

void
xmpp_roster_request_info (xmpp_roster_t roster,
                          const char *jid)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->RequestInfo (buzz::Jid (jid));
}

void
xmpp_roster_add_contact (xmpp_roster_t roster,
                         const char *jid,
                         const char *name,
                         const char *group)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->AddContact (buzz::Jid (jid), std::string (name), std::string (group));
}

void
xmpp_roster_remove_contact (xmpp_roster_t roster,
                            const char *jid)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->RemoveContact (buzz::Jid (jid));
}

void
xmpp_roster_set_contact_denied_cb (xmpp_roster_t roster,
                                   xmpp_roster_deny_authorize_cb_type cb,
                                   void *data)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->SetDenyCb (cb, data);
}

void
xmpp_roster_set_contact_authorized_cb (xmpp_roster_t roster,
                                       xmpp_roster_deny_authorize_cb_type cb,
                                       void *data)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->SetAuthorizeCb (cb, data);
}

void
xmpp_roster_set_authorization_requested_cb (xmpp_roster_t roster,
                                            xmpp_roster_deny_authorize_cb_type cb,
                                            void *data)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->SetRequestCb (cb, data);
}

void
xmpp_roster_set_contact_retrieved_cb (xmpp_roster_t roster,
                                      xmpp_roster_contact_retrieved_cb_type cb,
                                      void *data)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->SetContactCb (cb, data);
}

void
xmpp_roster_set_info_retrieved_cb (xmpp_roster_t roster,
                                   xmpp_roster_info_retrieved_cb_type cb,
                                   void *data)
{
    XmppRoster *xr = reinterpret_cast<XmppRoster *>(roster);
    xr->SetInfoCb (cb, data);
}
